import logging
import re
from typing import AsyncGenerator

import httpx
from fastapi import HTTPException

from llm_bridge.client.implementations.claude.claude_token_counter import count_claude_input_tokens, \
    count_claude_output_tokens
from llm_bridge.client.model_client.claude_client import ClaudeClient
from llm_bridge.type.chat_response import ChatResponse
from llm_bridge.type.serializer import serialize


class PrintingStatus:
    Start = "start"
    Thought = "thought"
    Response = "response"


class StreamClaudeClient(ClaudeClient):
    async def generate_stream_response(self) -> AsyncGenerator[ChatResponse, None]:
        try:
            logging.info(f"messages: {self.messages}")

            try:
                async with self.client.beta.messages.stream(
                    model=self.model,
                    max_tokens=self.max_tokens,
                    temperature=self.temperature,
                    system=self.system,
                    messages=serialize(self.messages),
                    thinking=self.thinking,
                    betas=self.betas,
                ) as stream:
                    printing_status = PrintingStatus.Start

                    async for event in stream:
                        text = ""
                        if event.type == "content_block_start":
                            continue
                        elif event.type == "content_block_delta":
                            if event.delta.type == "thinking_delta":
                                if printing_status == PrintingStatus.Start:
                                    text += "# Model Thought:\n\n"
                                    printing_status = PrintingStatus.Thought
                                text += event.delta.thinking
                            elif event.delta.type == "text_delta":
                                if printing_status == PrintingStatus.Thought:
                                    text += "\n\n# Model Response:\n\n"
                                    printing_status = PrintingStatus.Response
                                text += event.delta.text

                        chat_response = ChatResponse(text=text)
                        output_tokens = await count_claude_output_tokens(
                            client=self.client,
                            model=self.model,
                            chat_response=chat_response,
                        )
                        yield ChatResponse(
                            text=text,
                            input_tokens=self.input_tokens,
                            output_tokens=output_tokens,
                        )

            except Exception as e:
                logging.exception(e)
                yield ChatResponse(error=repr(e))

        except httpx.HTTPStatusError as e:
            status_code = e.response.status_code
            text = e.response.text
            raise HTTPException(status_code=status_code, detail=text)
        except Exception as e:
            logging.exception(e)
            match = re.search(r'\d{3}', str(e))
            if match:
                error_code = int(match.group(0))
            else:
                error_code = 500

            raise HTTPException(status_code=error_code, detail=str(e))



import anthropic
from anthropic.types import ThinkingConfigEnabledParam

from llm_bridge.client.implementations.claude.claude_token_counter import count_claude_input_tokens
from llm_bridge.client.implementations.claude.non_stream_claude_client import NonStreamClaudeClient
from llm_bridge.client.implementations.claude.stream_claude_client import StreamClaudeClient
from llm_bridge.logic.chat_generate.chat_message_converter import convert_messages_to_claude
from llm_bridge.logic.message_preprocess.message_preprocessor import extract_system_messages
from llm_bridge.type.message import Message


async def create_claude_client(
        messages: list[Message],
        model: str,
        temperature: float,
        stream: bool,
        api_key: str,
):
    client = anthropic.AsyncAnthropic(
        api_key=api_key,
    )

    system = extract_system_messages(messages)
    if system == "":
        system = "/"

    claude_messages = await convert_messages_to_claude(messages)

    input_tokens = await count_claude_input_tokens(
        client=client,
        model=model,
        system=system,
        messages=claude_messages,
    )

    max_tokens = min(128000, 200000 - input_tokens)
    thinking = ThinkingConfigEnabledParam(
        type="enabled",
        budget_tokens=32000
    )
    temperature = 1
    betas = ["output-128k-2025-02-19"]

    if stream:
        return StreamClaudeClient(
            model=model,
            messages=claude_messages,
            temperature=temperature,
            system=system,
            client=client,
            max_tokens=max_tokens,
            thinking=thinking,
            betas=betas,
            input_tokens=input_tokens,
        )
    else:
        return NonStreamClaudeClient(
            model=model,
            messages=claude_messages,
            temperature=temperature,
            system=system,
            client=client,
            max_tokens=max_tokens,
            thinking=thinking,
            betas=betas,
            input_tokens=input_tokens,
        )





from typing import AsyncGenerator, Any

import anthropic
from anthropic.types import ThinkingConfigEnabledParam, AnthropicBetaParam

from llm_bridge.client.chat_client import ChatClient
from llm_bridge.type.chat_response import ChatResponse
from llm_bridge.type.model_message.claude_message import ClaudeMessage


class ClaudeClient(ChatClient):
    def __init__(
            self,
            model: str,
            messages: list[ClaudeMessage],
            temperature: float,
            system: str,
            client: anthropic.AsyncAnthropic,
            max_tokens: int,
            thinking: ThinkingConfigEnabledParam,
            betas: list[AnthropicBetaParam],
            input_tokens: int,
    ):
        self.model = model
        self.messages = messages
        self.temperature = temperature
        self.system = system
        self.client = client
        self.max_tokens = max_tokens
        self.thinking = thinking
        self.betas = betas
        self.input_tokens = input_tokens

    async def generate_non_stream_response(self) -> ChatResponse:
        raise NotImplementedError
    async def generate_stream_response(self) -> AsyncGenerator[ChatResponse, None]:
        raise NotImplementedError



from dataclasses import dataclass
from enum import Enum

from anthropic.types import MessageParam
from typing_extensions import TypeAlias

from llm_bridge.type.message import Role


class ClaudeRole(Enum):
    User = "user"
    Assistant = "assistant"


ClaudeMessage: TypeAlias = MessageParam

# class MessageParam(TypedDict, total=False):
#     content: Required[
#         Union[
#             str,
#             Iterable[
#                 Union[
#                     TextBlockParam,
#                     ImageBlockParam,
#                     ToolUseBlockParam,
#                     ToolResultBlockParam,
#                     DocumentBlockParam,
#                     ThinkingBlockParam,
#                     RedactedThinkingBlockParam,
#                     ContentBlock,
#                 ]
#             ],
#         ]
#     ]
#
#     role: Required[Literal["user", "assistant"]]



from dataclasses import dataclass
from typing import Optional


@dataclass
class Citation:
    text: str
    indices: list[int]


@dataclass
class ChatResponse:
    text: Optional[str] = None
    image: Optional[str] = None
    display: Optional[str] = None
    citations: Optional[list[Citation]] = None
    error: Optional[str] = None
    input_tokens: Optional[int] = 0
    output_tokens: Optional[int] = 0



import pytest

from llm_bridge.logic.chat_generate.chat_client_factory import create_chat_client
from llm_bridge.type.message import Message, Role, Content, ContentType


@pytest.fixture
def sample_messages():
    return [
        Message(role=Role.System, contents=[
            Content(type=ContentType.Text, data="You are a helpful assistant.")
        ]),
        Message(role=Role.User, contents=[
            Content(type=ContentType.Text, data="Hello")
        ])
    ]


@pytest.mark.asyncio
async def test_placeholder():
    assert True
